/*
 * Copyright (c) 2011 Wolfgang Solfrank.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the  
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Wolfgang Solfrank.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__MiXen_h__
#define	__MiXen_h__

#include <types.h>

#include <config.h>

#include <xen.h>
#ifndef __ASSEMBLY__
#include <event_channel.h>
#include <grant_table.h>
#endif

#include <machine/param.h>

/*
 * Version number is encoded as 16 bit with high byte as major
 * and low byte as minor number.
 * Note: While we print the version number as major.minor, this is not
 * intended as a decimal number, as 1.11 is larger (later) than 1.9!
 */
#define	__MiXen__	0x0003

/* Be sure to evaluate "cond" even if we are not checking the condition!!! */
#define	ASSERT(cond)	do {					\
			    int __assert;			\
								\
			    __assert = (cond) != 0;		\
			    if (!__assert)			\
				failure(__func__, #cond);	\
			} while (0)

#ifndef	__ASSEMBLY__
extern const char version[];

extern start_info_t *start_info;
extern shared_info_t *shared_info;

extern long hypercall_callback_op(int, void *);
extern int hypercall_console_io(int, int, char *);
extern int hypercall_event_channel_op(int, void *);
extern int hypercall_grant_table_op(int, void *, int);
extern int hypercall_mmu_update(mmu_update_t *, unsigned int, 
				unsigned int *, domid_t);
extern int hypercall_mmuext_op(mmuext_op_t *, unsigned int,
				unsigned int *, domid_t);
extern int hypercall_sched_op(int, void *);
extern int hypercall_vcpu_op(int, int, void *);

extern void init_hypercall_page(void);
extern void switchstack(void *);
extern void install_handler(evtchn_port_t, void (*)(void *), void *);
extern void uninstall_handler(evtchn_port_t);

extern void halt(void) __attribute__((noreturn));
extern void yield(void);
extern void block(void);
extern void pollevents(uint32_t);

extern void find_vcpus(void);
extern void run_vcpu(void);

extern void init_console(void);
extern void console_write(void *, const char *, size_t);

extern void init_xenstore(void);
extern domid_t xs_domid(void *);
#ifdef	STDARG_FIXED
extern void xs_write(void *, const char *, const char *, ...);
#else
extern void xs_uint64(void *, const char *, uint64_t);
#endif

/*
 * backend_setparams returns some opaque cookie
 * to be presented to any other backend_* routines.
 */
extern void *backend_setparams(void *);
extern void backend_setup(void *, domid_t, grant_ref_t, evtchn_port_t, int);
extern void backend_destroy(void *);

/* I/O Requests under Xen are in these units: */
#define	XEN_BSIZE	512

#ifdef	DEBUG
extern int debug;
#define	DBG_ALL		(-1)
#define	DBG_XS		1
#define	DBG_BACKEND	2
#define	DPRINTF(subsys, format, ...)	do {\
					    if (debug & (subsys))\
						printf(format, ## __VA_ARGS__);\
					} while (0)
#else
#define	DPRINTF(subsys, format, ...)
#endif
#endif

#endif
