/*
 * Copyright (c) 2011 Wolfgang Solfrank.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the  
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Wolfgang Solfrank.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__MEMORY_H__
#define	__MEMORY_H__

#include <grant_table.h>

/*
 * Some definitions for the memory layout (including page table management).
 */

/* Initial alignment & gap. See description of start_info */
#define	SOD_ALIGNMENT	(4*1024*1024)
#define	SOD_GAP		(512*1024)

extern void setup_mem(void);
extern void init_mem(void);

/* Allocate a page of memory */
extern void *malloc(size_t);
extern void free(void *);

/* Map/unmap a given mfn. */
extern void *mapforeign(domid_t, grant_ref_t, grant_handle_t *, uint64_t *);
extern void unmapforeign(grant_handle_t, void *, uint64_t);

/* convert some virtual address into an mfn. */
extern uint64_t vaddr2mfn(void *);

/* Number of pages available. */
extern uint64_t freecnt;

#ifdef	RAMDISK
extern uint64_t disksize;
extern void *ramdisk;
#endif

#endif
